import * as THREE from '../build/three.module.js';
import { TrackballControls } from './jsm/controls/TrackballControls.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let scene = new THREE.Scene();
        scene.background = new THREE.Color(0x222222);

        let renderer = new THREE.WebGLRenderer({ antialias: true });
        this.domWebGL.appendChild(renderer.domElement);

        this.renderer = renderer;
        this.scene = scene;

        this._setupCamera();
        this._setupModel();
        this._setupLights();
        this._setupControls();

        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupControls() {
        const controls = new TrackballControls(this._camera, this.renderer.domElement);
        controls.minDistance = 200;
        controls.maxDistance = 500;

        this._controls = controls;
    }

    _setupModel() {
        // First Mesh
        const closedSpline = new THREE.CatmullRomCurve3([
            new THREE.Vector3(-60, -100, 60),
            new THREE.Vector3(-60, 20, 60),
            new THREE.Vector3(-60, 120, 60),
            new THREE.Vector3(60, 20, -60),
            new THREE.Vector3(60, -100, -60)
        ]);

        closedSpline.curveType = 'catmullrom';
        closedSpline.closed = true;

        const extrudeSetting1 = {
            steps: 100,
            bevelEnabled: false,
            extrudePath: closedSpline
        };

        const pts1 = [], count = 3;
        for(let i=0; i<count; i++) {
            const l = 20;
            const a = 2*i/count*Math.PI;
            pts1.push(new THREE.Vector2(Math.cos(a)*l, Math.sin(a)*l));
        }

        const shape1 = new THREE.Shape(pts1);
        const geometry1 = new THREE.ExtrudeGeometry(shape1, extrudeSetting1);
        const material1 = new THREE.MeshLambertMaterial({ color: 0xb00000, wireframe: false });
        const mesh1 = new THREE.Mesh(geometry1, material1);
        this.scene.add(mesh1);

        // Second Mesh
        const randomPoints = [];
        for(let i=0; i<10; i++) {
            randomPoints.push(
                new THREE.Vector3(
                    (i-4.5)*50, 
                    THREE.MathUtils.randFloat(-50, 50),
                    THREE.MathUtils.randFloat(-50, 50) 
                )
            );
        }

        const randomSpline = new THREE.CatmullRomCurve3(randomPoints);
        const extrudeSetting2 = {
            steps: 200,
            bevelEnabled: false,
            extrudePath: randomSpline
        };

        const pts2 = [], numPts = 5;
        for(let i=0; i<numPts*2; i++) {
            const l = i%2 == 1 ? 10 : 20;
            const a = i / numPts * Math.PI;
            pts2.push(new THREE.Vector2(Math.cos(a)*l, Math.sin(a)*l));
        }
        const shape2 = new THREE.Shape(pts2);
        const geometry2 = new THREE.ExtrudeGeometry(shape2, extrudeSetting2);
        const material2 = new THREE.MeshLambertMaterial({ color: 0xff8000, wireframe: false });
        const mesh2 = new THREE.Mesh(geometry2, material2);
        this.scene.add(mesh2);

        // Third Mesh
        const materials = [material1, material2];
        const extrudeSetting3 = {
            depth: 60,
            steps: 1,
            bevelEnabled: true,
            bevelThickness: 2,
            bevelSize: 4,
            bevelSegments: 1
        };

        const geometry3 = new THREE.ExtrudeGeometry(shape2, extrudeSetting3);
        const mesh3 = new THREE.Mesh(geometry3, materials);
        mesh3.position.set(50, 100, 50);
        this.scene.add(mesh3);

    }

    _setupLights() {
        this.scene.add(new THREE.AmbientLight(0x222222));

        const light = new THREE.PointLight(0xffffff);
        light.position.copy(this._camera.position);
        this.scene.add(light);
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 1, 1000);
        camera.position.set(0, 0, 500);
        this.scene.add(camera);

        this._camera = camera;
    }

    update() {
        this._controls.update();
    }

    render(time) {
        requestAnimationFrame(this.render.bind(this));

        this.update(time);

        this.renderer.render(this.scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this.render();
    }
}

window.onload = function () {
    (new App()).render();
}